"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }





var _chunk5DS3KGP5js = require('./chunk-5DS3KGP5.js');

// src/core/index.ts
var _unplugin = require('unplugin');
var _pluginutils = require('@rollup/pluginutils');

// src/core/parse_ID.ts
var _path = require('path');
function parse_ID(id, root = "") {
  const [file, rawQuery] = id.split("?", 2);
  const ext = _path.extname.call(void 0, file).slice(1);
  if (ext === "vue") {
    return {
      file: file.replace(root, ""),
      isSfc: true,
      query: Object.fromEntries(new URLSearchParams(rawQuery))
    };
  }
  return {
    file: file.replace(root, ""),
    isJsx: true,
    isTsx: ext.includes("ts")
  };
}

// src/core/transform.ts
var _magicstring = require('magic-string'); var _magicstring2 = _interopRequireDefault(_magicstring);

// src/core/transform_SFC.ts
var _compilerdom = require('@vue/compiler-dom');

// src/core/transform_JSX.ts
var _core = require('@babel/core');
var _parser = require('@babel/parser');
function transform_JSX(code, cb, options) {
  const { isTsx, startIndex = 0, startLine = 1, startColumn = 1 } = options;
  const plugins = ["jsx"];
  if (isTsx) {
    plugins.push("typescript");
  }
  const ast = _parser.parse.call(void 0, code, {
    sourceType: "unambiguous",
    plugins,
    startLine,
    // babel start at 0
    startColumn: startColumn - 1
  });
  _core.traverse.call(void 0, ast, {
    JSXOpeningElement({ node }) {
      const nameNode = node.name;
      if (!nameNode)
        return;
      const { start } = node.loc;
      const name = getJSXElementName(nameNode);
      const offset = start.index + startIndex + name.length + 1;
      cb(_chunk5DS3KGP5js.__spreadProps.call(void 0, _chunk5DS3KGP5js.__spreadValues.call(void 0, {}, start), {
        // babel starts at 0, so we need to add 1
        column: start.column + 1,
        offset
      }));
    }
  });
}
function getJSXElementName(nameNode) {
  let nameValue;
  if (_core.types.isJSXIdentifier(nameNode)) {
    nameValue = nameNode.name;
  } else if (_core.types.isJSXNamespacedName(nameNode)) {
    const { namespace, name } = nameNode;
    nameValue = `${namespace.name}:${name.name}`;
  } else {
    const nameValues = [];
    while (_core.types.isJSXMemberExpression(nameNode)) {
      nameValues.unshift(nameNode.property.name);
      nameNode = nameNode.object;
    }
    nameValues.unshift(nameNode.name);
    nameValue = nameValues.join(".");
  }
  return nameValue;
}

// src/core/transform_SFC.ts
function transform_SFC(code, cb) {
  const ast = _compilerdom.parse.call(void 0, code);
  _compilerdom.transform.call(void 0, ast, {
    nodeTransforms: [
      (node) => {
        if (node.type === _chunk5DS3KGP5js.NodeTypes.ELEMENT && _chunk5DS3KGP5js.TagTypes.includes(node.tagType)) {
          const { start } = node.loc;
          const offset = start.offset + node.tag.length + 1;
          cb(_chunk5DS3KGP5js.__spreadProps.call(void 0, _chunk5DS3KGP5js.__spreadValues.call(void 0, {}, start), { offset }));
        }
      }
    ]
  });
  const jsxOpts = resolveJsxOptions(ast);
  if (jsxOpts) {
    transform_JSX(jsxOpts.code, cb, jsxOpts);
  }
}
function resolveJsxOptions(ast) {
  var _a;
  const scriptNode = ast.children.find(
    (node) => node.tag === "script"
  );
  if (!scriptNode)
    return;
  const codeNode = scriptNode.children[0];
  if (!codeNode)
    return;
  const langProp = scriptNode.props.find(
    (prop) => prop.name === "lang"
  );
  if (!langProp)
    return;
  const lang = (_a = langProp.value) == null ? void 0 : _a.content;
  const isTsx = lang === "tsx";
  const isJsx = isTsx || lang === "jsx";
  if (isJsx) {
    const { offset, line, column } = codeNode.loc.start;
    return {
      isTsx,
      startIndex: offset,
      startLine: line,
      startColumn: column,
      code: codeNode.content
    };
  }
}

// src/core/transform.ts
function transform2(code, id, options) {
  const { root, sourceMap } = options;
  const s = new (0, _magicstring2.default)(code);
  const parsed = parse_ID(id, root);
  if (parsed.isSfc) {
    transform_SFC(code, replace);
  } else {
    transform_JSX(code, replace, parsed);
  }
  function replace(pos) {
    const { offset, line, column } = pos;
    s.prependLeft(offset, ` ${_chunk5DS3KGP5js.TRACE_ID}="${parsed.file}:${line}:${column}"`);
  }
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}

// src/core/index.ts
var unpluginFactory = (options = {}) => {
  if (process.env.NODE_ENV !== "development") {
    return {
      name: "unplugin-vue-source"
    };
  }
  const opts = resolveOptions(options);
  const filter = _pluginutils.createFilter.call(void 0, opts.include, opts.exclude);
  return {
    name: "unplugin-vue-source",
    enforce: "pre",
    transformInclude(id) {
      const { file, isSfc, query } = parse_ID(id);
      if (filter(file)) {
        if (isSfc && query.type !== "template") {
          return (
            // vite-plugin-vue
            !query[_chunk5DS3KGP5js.TRACE_ID] && // rollup-plugin-vue
            !query["rollup-plugin-vue"]
          );
        }
        return true;
      }
    },
    transform(code, id) {
      return transform2(code, id, opts);
    }
  };
};
function resolveOptions(opts) {
  var _a, _b, _c, _d;
  return {
    root: (_a = opts.root) != null ? _a : process.cwd(),
    sourceMap: (_b = opts.sourceMap) != null ? _b : false,
    include: (_c = opts.include) != null ? _c : "**/*.{vue,jsx,tsx}",
    exclude: (_d = opts.exclude) != null ? _d : "node_modules/**"
  };
}
var core_default = /* @__PURE__ */ _unplugin.createUnplugin.call(void 0, unpluginFactory);



exports.core_default = core_default;
